
subroutine CalcEnergy(Pos, Types, emat, kx, dx, M, k, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:1,0:1) :: emat
    integer, intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in) :: L, rc, k
    real(8), intent(out) :: PEnergy
!f2py intent(in, out) :: Forces
    real(8), parameter :: r0 = 1.
    real(8), dimension(0:Dim-1) ::  rvij, Posi
    real(8)                     ::  shift, id2, r2ij, id12, id6, rij, r2c, kx, dx
    integer                     ::  i, j, Typei, Typej

PEnergy = 0
shift = 4*(rc**(-12.0)-rc**(-6.0))
r2c = rc*rc

PEnergy = PEnergy + kx/2.0*dx*dx

do i = 0, NAtom - 1 
    !Copy the temp trick from ex2lib.f90
    Posi = Pos(i,:)
    Typei = Types(i)

    !Every atom interacts with the wall
    !First bottom:
    rvij = 0.
    rvij(0) = -L - Posi(0)

    r2ij = rvij(0)*rvij(0)
    id2 = 1. / r2ij
    id6 = id2 * id2 * id2
    id12 = id6 * id6

    PEnergy = PEnergy + 4*(id12)

    !Then top:
    rvij = 0.
    rvij(0) = L - Posi(0)

    r2ij = rvij(0)*rvij(0)
    id2 = 1. / r2ij
    id6 = id2 * id2 * id2
    id12 = id6 * id6

    PEnergy = PEnergy + 4*(id12)
    do j = i + 1, NAtom - 1 
        Typej = Types(j)

        rvij = Pos(j,:) - Posi
        rvij(1) = rvij(1) - L * dnint(rvij(1) / L)
        rvij(2) = rvij(2) - L * dnint(rvij(2) / L)
        !rvij = rvij - L * dnint(rvij / L)
        r2ij = sum(rvij*rvij)

        if (j == i + 1 .and. mod(j, M) > 0 .and. Typei == 0 .and. Typej == 0) then 
            !this is a bonded interaction; 
            !use the harmonic bond potential here 
            rij = sqrt(r2ij)
            
            PEnergy = PEnergy + k/2.0*(rij-r0)*(rij-r0)
        elseif (r2ij < r2c) then
            !this is a nonbonded interaction; 
            !use the Lennard-Jones potential here 
            id2 = 1. / r2ij
            id6 = id2 * id2 * id2
            id12 = id6 * id6

            PEnergy = PEnergy + 4*emat(Typei,Typej)*((id12-id6)-shift)
        endif 
    enddo 
enddo

!PEnergy = PEnergy - UTail

end subroutine

subroutine CalcEnergyForces(Pos, Types, emat, kx, dx, M, k, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in), dimension(0:1,0:1) :: emat
    real(8), intent(in) :: L, rc, k
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: r0 = 1.
    real(8), dimension(0:Dim-1) ::  rvij, Posi, Fij
    real(8)                     ::  shift, id2, r2ij, id12, id6, rij, r2c, kx, dx
    integer                     ::  i, j, Typei, Typej

PEnergy = 0
Forces = 0.
shift = 4*(rc**(-12.0)-rc**(-6.0))
r2c = rc*rc

!Biased sampling potential
PEnergy = PEnergy + kx/2.0*dx*dx

do i = 0, NAtom - 1 
    !Copy the temp trick from ex2lib.f90
    Posi = Pos(i,:)
    Typei = Types(i)

    !Every atom interacts with the wall
    !First bottom:
    rvij = 0.
    rvij(0) = -L - Posi(0)

    r2ij = rvij(0)*rvij(0)
    id2 = 1. / r2ij
    id6 = id2 * id2 * id2
    id12 = id6 * id6

    PEnergy = PEnergy + 4*(id12)
    Fij = 0.
    Fij(0) = 48*id12
    Forces(i,:) = Forces(i,:) + Fij

    !Then top:
    rvij = 0.
    rvij(0) = L - Posi(0)

    r2ij = rvij(0)*rvij(0)
    id2 = 1. / r2ij
    id6 = id2 * id2 * id2
    id12 = id6 * id6

    PEnergy = PEnergy + 4*(id12)
    Fij = 0.
    Fij(0) = -48*id12

    Forces(i,:) = Forces(i,:) + Fij

    if (i<M) then
        Forces(i,0) = Forces(i,0) + kx*dx/M
    endif

    do j = i + 1, NAtom - 1 
        Typej = Types(j)

        rvij = Pos(j,:) - Posi
        rvij(1) = rvij(1) - L * dnint(rvij(1) / L)
        rvij(2) = rvij(2) - L * dnint(rvij(2) / L)
        !rvij = rvij - L * dnint(rvij / L)

        r2ij = sum(rvij*rvij)

       ! print*, i, j

        if (j == i + 1 .and. mod(j, M) > 0 .and. Typei == 0 .and. Typej == 0) then 
            !this is a bonded interaction; 
            !use the harmonic bond potential here 
            rij = sqrt(r2ij)
            Fij = k*(rij-r0)
            Fij = Fij*rvij/rij

           ! print*, Fij
           ! print*, k/2.0*(rij-r0)*(rij-r0)

            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
            
            PEnergy = PEnergy + k/2.0*(rij-r0)*(rij-r0)
        elseif (r2ij < r2c) then
            !this is a nonbonded interaction; 
            !use the Lennard-Jones potential here 
            id2 = 1. / r2ij

            id6 = id2 * id2 * id2
            id12 = id6 * id6

            Fij = emat(Typei,Typej)*(-48*id12 + 24*id6)
            Fij = Fij * id2
            Fij = Fij * rvij

            Forces(i,:) = Forces(i,:) + Fij 
            Forces(j,:) = Forces(j,:) - Fij
            PEnergy = PEnergy + 4*emat(Typei,Typej)*((id12-id6) - shift)
        endif 
    enddo 
enddo

end subroutine

subroutine VVIntegrate(Pos, Types, Vel, Accel, emat, kx, dx, M, k, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt, k, kx, dx
    real(8), intent(in), dimension(0:1,0:1) :: emat
    integer, intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Types, emat, kx, dx, M, k, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine
